.org        $9000

; Write the FPGA configuration in the P6M2VGA1 and P6M2VGA2 disks
; to the P6 mkII VGA adapter.
;
; SPI connection
;
; JOYSTICK 2
; pin 1 : A0 = MISO
; pin 6 : B0 = SS
; pin 7 : B1 = MOSI
; pin 8 : B4 = CLK
;
; 2016/6/21 Koichi Nishida

	jp	START

; initialize SPI
SPI_init:
	ld	a,7
    out	($a0),a
    ld	a,$80
    out	($a1),a
    
    ld	a,$f
   	out	($a0),a
    ld	a,%00000001
    out	($a1),a
    ret

; restore SPI
SPI_end:
	ld a,7
    out ($a0),a
    ld	a,0
    out	($a1),a
    
    ld	a,$f
   	out	($a0),a
    ld	a,%10000000
    out	($a1),a
    
    ret
    
; enable SS signal
SPI_enableSS:
	push af
    
    ld	a,7
    out	($a0),a
    ld	a,$80
    out	($a1),a
    
	ld	a,$f
   	out	($a0),a
	ld	a,%00000000
    out ($a1),a
    pop af
    ret

; disable SS signal
SPI_disableSS:
	push af
    
    ld	a,7
    out	($a0),a
    ld	a,$80
    out	($a1),a
    
	ld	a,$f
   	out	($a0),a
	ld	a,%00000001
    out ($a1),a
    pop af
    ret

; write the byte data in the A register to the SPI
SPI_readWriteByte:
	push	de
    push	bc
	ld	b,8
	ld	c,a
    ld	e,0
SPI_wb_loop:
	rlc	c
    jr	nc,SPI_wb_0
    
    ld	a,$f
   	out	($a0),a
	ld	a,%00000010
    out ($a1),a
    ld	a,%00010010
    out	($a1),a
    jr	SPI_wb_merge
SPI_wb_0:

	ld	a,$f
   	out	($a0),a
	ld	a,%00000000
    out ($a1),a
    ld	a,%00010000
    out	($a1),a
SPI_wb_merge:

	ld	a,$e
	out	($a0),a
	in	a,($a2)
	rrca
    rl	e

	djnz SPI_wb_loop
    ld	a,e
    pop bc
    pop	de
    ret
    
; begin SPI FLASH memory write    
;    HL; page address
SPI_FLASH_writeBegin:
	call SPI_enableSS
    ld a,6
    call SPI_readWriteByte
    call SPI_disableSS
    call SPI_enableSS
    ld a,2
    call SPI_readWriteByte
    ld a,h
    call SPI_readWriteByte
    ld a,l
    call SPI_readwriteByte
    ld a,0
    call SPI_readWriteByte
    ret

; end SPI FLASH memory read
SPI_FLASH_writeEnd:
	call	SPI_disableSS
SPI_FLASH_we_lp:
	call	SPI_enableSS
	ld a,5
    call SPI_readWriteByte
    ld a,$ff
    call SPI_readWriteByte
    call SPI_disableSS
    and	1
    jr nz,SPI_FLASH_we_lp
    
    call SPI_enableSS
    ld a,4
    call SPI_readWriteByte
    call SPI_disableSS
    
    ret

; begin SPI FLASH memory read
;    HL: page address    
SPI_FLASH_readBegin:
	call SPI_enableSS
    ld a,3
    call SPI_readWriteByte
    ld a,h
    call SPI_readWriteByte
    ld a,l
    call SPI_readwriteByte
    ld a,0
    call SPI_readWriteByte
    ret    

; end SPI FLASH memory read
SPI_FLASH_readEnd:
 	call	SPI_disableSS
    ret

; erase SPI FLASH memory
SPI_FLASH_erase:	
	call SPI_enableSS
    ld a,6
    call SPI_readWriteByte
    call SPI_disableSS
	call SPI_enableSS
    ld a,$c7
    call SPI_readWriteByte
    call SPI_disableSS
SPI_FLASH_erase_lp:
	 call	SPI_enableSS
	ld a,5
    call SPI_readWriteByte
    ld a,$ff
    call SPI_readWriteByte
    call SPI_disableSS
    and	1
    jr nz,SPI_FLASH_erase_lp
    call SPI_enableSS
    ld a,4
    call SPI_readWriteByte
    call SPI_disableSS
    
    ret
    
; print a string
PRINT:
	ld a,(hl)
    or a
    ret z
    call $1075  ; print char
    inc hl
    jr PRINT

; compare two strings
; if the same, carry flag is set
STRCMP:
	ld a,(de)
    or a
    jr z,STRCMP_same
   	cp (hl)
    jr	nz,STRCMP_diff
    inc	de
    inc hl
    jr STRCMP
STRCMP_same:
	scf
    ret
STRCMP_diff:
	or a
    ret

; write a configuration to the SPI FLASH memory
WRITE_CONFIG:
	ld hl,0 ; page address
    ld (PAGEADR),hl
    ld a,0
    ld (FDNUM),a
    
WC_LOOP:
	ld hl,MES_insertFD1
    call PRINT
    
    ld a,(FDNUM)
    add a,$31
    ld (STR_FD+7),a
    
    call $1075  ; print char
    
    ld hl,MES_insertFD2
    call PRINT
    
	call $0fc4 ; wait a key
    
     ; The size of FPGA configuration data is about 146 KB.
    ; So we use two disks.
    ; We use track 1 - 19 (track 0 is a header).
    
    ld a, 0
    ld ($f9d2),a
	ld ($f9d2 + 27),a
    ld a,1
    ld b,0
    ld c,1
    ld de,BUFFER
   	or a
    call $4274

	ld de,STR_FD
    ld hl,BUFFER
    call STRCMP
    jr c,WC_FDOK
    
    ld hl,MES_FDNG
    call PRINT
    
    jr WC_LOOP
    
WC_FDOK:
	ld b,1 ; track

WC_TRACK_LP:

    push	bc   
    ld	h,0
    ld	l,b
    call	$3aa1
    pop bc
  
  	ld hl,MES_DIV19
	call PRINT

	ld c,1 ; sector
WC_SECTOR_LP:
	push bc
    
	ld a, 0
    ld ($f9d2),a
	ld ($f9d2 + 27),a
    ld a,1 ; 1sector
    ld de,BUFFER
   	or a
    call $4274
    
    ld hl,(PAGEADR)
	call SPI_FLASH_writeBegin
    
    ld b,0
    ld hl,BUFFER
WC_writeBytes:
	ld a,(hl)
    call SPI_readWriteByte
    inc hl
    djnz WC_writeBytes
    
    call SPI_FLASH_writeEnd
    
    ld hl,(PAGEADR)
    inc hl
    ld (PAGEADR),hl
    
	pop bc
    
    inc c
    ld a,17
    cp c
    jr nz,WC_SECTOR_LP

	inc b
    ld a,20
    cp b
    jr nz,WC_TRACK_LP
    
    ld a,$0d
	call $1075  ; print char
    ld a,$0a
	call $1075  ; print char
    
    ld a,(FDNUM)
    inc a
    ld (FDNUM),a
    cp 2
    jp nz,WC_LOOP
    
	ld a,$0d
	call $1075  ; print char
    
    ; SUCCESS!
    or a
	ret

; verify the configuration in the SPI FLASH memory
VERIFY_CONFIG:
	ld hl,0 ; page address
    ld (PAGEADR),hl
    ld a,0
    ld (FDNUM),a
    
VC_LOOP:
	
	ld hl,MES_insertFD1
    call PRINT
    
    ld a,(FDNUM)
    add a,$31
    
    ld (STR_FD+7),a
    
    call $1075  ; print char
    
    ld hl,MES_insertFD2
    call PRINT
    
	call $0fc4 ; wait a key
    
    ; The size of FPGA configuration data is about 146 KB.
    ; So we use two disks.
    ; We use track 1 - 19 (track 0 is a header).
    
    ld a, 0
    ld ($f9d2),a
	ld ($f9d2 + 27),a
    ld a,1
    ld b,0
    ld c,1
    ld de,BUFFER
   	or a
    call $4274

	ld de,STR_FD
    ld hl,BUFFER
    call STRCMP
    jr c,VC_FDOK
    
    ld hl,MES_FDNG
    call PRINT
    
    jr VC_LOOP
     
VC_FDOK:
	ld b,1 ; track

VC_TRACK_LP:
    push	bc   
    ld	h,0
    ld	l,b
    call	$3aa1
    pop bc
  
	ld hl,MES_DIV19
	call PRINT
  
	ld c,1 ; sector
VC_SECTOR_LP:
	push bc
	ld a, 0
    ld ($f9d2),a
	ld ($f9d2 + 27),a
    ld a,1 ; 1sectors
    ld de,BUFFER
   	or a
    call $4274
    
    ld hl,(PAGEADR)
	call SPI_FLASH_readBegin
    
    ld b,0
    ld hl,BUFFER2
VC_readBytes:
	ld a,$ff
    call SPI_readWriteByte
    ld (hl),a
    inc hl
    djnz VC_readBytes
    
    call SPI_FLASH_readEnd

    ld b,0
    ld hl,BUFFER2
    ld de,BUFFER
VC_lp:
    ld a,(de)
    cp (hl)
    
    jr z,VC_OK
    ; ERROR!
    
	pop bc
	scf
    ret

VC_OK:
	inc de
    inc hl
	djnz VC_lp
    
	pop bc
    
    ld hl,(PAGEADR)
    inc hl
    ld (PAGEADR),hl
    
    inc c
    ld a,17
    cp c
    jr nz,VC_SECTOR_LP

	inc b
    ld a,20
    cp b
    jp nz,VC_TRACK_LP

	ld a,$0d
	call $1075  ; print char
    ld a,$0a
	call $1075  ; print char

	ld a,(FDNUM)
    inc a
    ld (FDNUM),a
    cp 2
    jp nz,VC_LOOP

	ld a,$0d
	call $1075  ; print char
  ; SUCCESS!
    or a
	ret

START:
	push	bc
    push	de
    push	hl
    push	af
   	di

	; clear screen
	ld a,$0c
	call $1075  ; print char

	call SPI_init
    ld hl,MES_instruction
    call PRINT
    
    call $0fc4 ; wait a key
    
    call SPI_FLASH_erase
    ld hl,MES_erased
    call PRINT

	call WRITE_CONFIG
    jr	c,ERROR
    
	ld hl,MES_VERIFY
    call PRINT
	call $0fc4 ; wait a key
	cp $59
    jr z,DO_VERIFY
    cp $79
    jr nz,SKIP_VERIFY
DO_VERIFY:
	call VERIFY_CONFIG
    jr c,ERROR
SKIP_VERIFY:
	ld hl,MES_SUCCESS
    call PRINT
    jr	MERGE
ERROR:
	ld hl,MES_ERROR
    call PRINT
MERGE:
    call SPI_end
    
    ei
    pop	af
    pop	hl
    pop	de
    pop	bc
    ret

STR_FD:
	.db "P6M2VGA ",0
MES_instruction:
	.db "Turn the PROG switch RIGHT, restart the P6 mkII VGA adapter and hit any key.",$0d,$0a,0
MES_erased:
	.db "The FLASH memory has been erased.",$0d,$0a,0
MES_insertFD1:
	.db "Insert P6M2VGA", 0
MES_insertFD2:
	.db " FDD and hit any key.",$0d,$0a,0
MES_FDNG:
	.db "Wrong FDD.", $0d,$0a,0
MES_ERROR:
	.db "ERROR!",$0d,$0a,0
MES_SUCCESS:
	.db "SUCCESS!",$0d,$0a,0    
MES_VERIFY:
	.db "Verify?",$0d,$0a,0
MES_DIV19:
	.db "/19",$0d,0
    
PAGEADR:
	.db 0,0
FDNUM:
	.db 0

BUFFER:
	.ds 256
BUFFER2:
	.ds 256

	

       
	

    
    
    
    
    
    

